/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.application.template.web.filter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jdesigner.platform.core.threadlocal.ThreadLocalProperties;
import org.jdesigner.platform.data.Tree;
import org.jdesigner.platform.util.StringUtils;

import org.jdesigner.application.template.domain.MenuLocation;
import org.jdesigner.application.template.domain.UserLogin;
import org.jdesigner.application.template.service.CommonServiceFactory;
import org.jdesigner.application.template.service.MenuService;
import org.jdesigner.application.template.service.SecurityService;
import org.jdesigner.application.template.web.WebKeys;

/**
 * <B> Menu Filter </B>
 * 
 * <pre> 1.사용자가 로그인할때 세션에 사용자가 접근 권한이 있는 메뉴 트리 맵을 설정한다. 2.URL에 매칭되는 메뉴를 조회하여, 이전/현재 선택 메뉴 및 메뉴의 경로를 설정한다. </pre>
 * 
 * @author leesangboo
 * 
 */
public class MenuFilter implements Filter {

	private static final Log log = LogFactory.getLog(MenuFilter.class);

	private static final String LOGIN_ERROR_PAGE_KEY = "login-error-page";
	private static final String ROLES_ERROR_PAGE_KEY = "roles-error-page";

	private String loginErrorPage;
	private String rolesErrorPage;

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
	 */
	public void init(FilterConfig config) throws ServletException {
		loginErrorPage = config.getInitParameter(LOGIN_ERROR_PAGE_KEY);
		rolesErrorPage = config.getInitParameter(ROLES_ERROR_PAGE_KEY);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.Filter#destroy()
	 */
	public void destroy() {
		//
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse, javax.servlet.FilterChain)
	 */
	public void doFilter(ServletRequest srequest, ServletResponse sresponse, FilterChain chain) throws IOException, ServletException {

		HttpServletRequest request = (HttpServletRequest) srequest;
		HttpServletResponse response = (HttpServletResponse) sresponse;
		HttpSession session = request.getSession();

		try {

			//
			String contextPath = request.getContextPath();
			String uri = request.getRequestURI().replace(request.getContextPath(), "");
			String fromuri = request.getRequestURI();
			String param = (StringUtils.isNotEmpty(request.getQueryString())) ? ("?" + request.getQueryString()) : "";

			//
			log.debug("Menu filtering start : url=[" + fromuri + param + "]");

			//
			MenuService menuService = CommonServiceFactory.getMenuService();
			SecurityService securityService = CommonServiceFactory.getSecurityService();

			// 1.사용자가 로그인할때 세션에 사용자가 접근 권한이 있는 메뉴 트리 맵을 설정한다.
			Tree userMenuTree = (Tree) session.getAttribute(WebKeys.USER_MENU_TREE_KEY);
			if (securityService.isUserAuthenticated() && userMenuTree == null) {
				UserLogin userLogin = securityService.getUserLogin();
				Map params = new HashMap();
				params.put("username", userLogin.getUsername());
				userMenuTree = menuService.selectMenuTreeByUsername(params);
				session.setAttribute(WebKeys.USER_MENU_TREE_KEY, userMenuTree);
				log.info("User's menu tree initialized : username=[" + userLogin.getUsername() + "]");
			}
			;

			// 1.이전 메뉴 경로 저장
			MenuLocation previousMenuLocation = (MenuLocation) session.getAttribute(WebKeys.USER_CURRENT_MENU_LOCATION_KEY);
			if (previousMenuLocation == null) {
				previousMenuLocation = new MenuLocation();
			}
			session.setAttribute(WebKeys.USER_PREVIOUS_MENU_LOCATION_KEY, previousMenuLocation);

			// 2.URL에 매칭되는 메뉴를 조회하여, 이전/현재 선택 메뉴 및 메뉴의 경로를 설정한다.
			MenuLocation menuLocation = menuService.getMenuLocationByURL(uri, param);
			Map menu = (menuLocation != null) ? menuLocation.getCurrentMenu() : null;

			//
			if (menu != null) {

				// 1.로그인 체크
				if (!securityService.isUserAuthenticated()) {
					session.setAttribute(WebKeys.USER_CURRENT_MENU_LOCATION_KEY, new MenuLocation());
					FilterUtils.redirectAndReturn(response, contextPath + loginErrorPage + "?redirect=" + fromuri + param);
				}

				// 2.메뉴 접근 롤 체크
				List menuRoleList = (List) menuLocation.getCurrentMenu().get("role_list");
				List checkMenuRoles = new ArrayList();
				if (menuRoleList != null && menuRoleList.size() > 0) {

					//로그인 체크
					if (!securityService.isUserAuthenticated()) {
						session.setAttribute(WebKeys.USER_CURRENT_MENU_LOCATION_KEY, new MenuLocation());
						FilterUtils.redirectAndReturn(response, contextPath + loginErrorPage + "?redirect=" + fromuri + param);
					}

					for (int i = 0; menuRoleList != null && i < menuRoleList.size(); i++) {
						checkMenuRoles.add(menuRoleList.get(i));
					}

					// 권한 체크
					if (securityService.isUserAuthenticated()) {
						boolean accessable = securityService.isUserInRoles(checkMenuRoles);
						//
						if (!accessable) {
							FilterUtils.redirectAndReturn(response, contextPath + rolesErrorPage);
						}
					} else {
						session.setAttribute(WebKeys.USER_CURRENT_MENU_LOCATION_KEY, new MenuLocation());
						FilterUtils.redirectAndReturn(response, contextPath + loginErrorPage + "?redirect=" + fromuri + param);
					}
				}
			}

			//
			if (session.getAttribute(WebKeys.USER_CURRENT_MENU_LOCATION_KEY) == null) {
				session.setAttribute(WebKeys.USER_CURRENT_MENU_LOCATION_KEY, new MenuLocation());
			}

			//
			if (menuLocation != null) {
				session.setAttribute(WebKeys.USER_CURRENT_MENU_LOCATION_KEY, menuLocation);
				if (menuLocation.getCurrentMenu() == null) {
					session.setAttribute(WebKeys.USER_CURRENT_MENU_LOCATION_KEY, new MenuLocation());
					FilterUtils.redirectAndReturn(response, contextPath + loginErrorPage + "?redirect=" + fromuri + param);
				}
			}

			log.debug("CurrentMenu set thread local properties.");
			ThreadLocalProperties.put(MenuService.MENU_LOCATION_KEY, session.getAttribute(WebKeys.USER_CURRENT_MENU_LOCATION_KEY));

			//
			chain.doFilter(srequest, sresponse);

		} catch (Exception ex) {
			log.error("Menu filter error : " + ex.getMessage(), ex);
			throw new ServletException(ex);
		}
	}

}
