/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.platform.db.processor.impl;

import java.sql.Connection;
import java.sql.ParameterMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.List;
import java.util.Map;

import org.apache.commons.dbutils.DbUtils;
import org.apache.commons.dbutils.ResultSetHandler;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jdesigner.platform.data.Page;
import org.jdesigner.platform.db.paging.PagingSQLHandler;
import org.jdesigner.platform.db.statement.NamedParameterStatement;
import org.jdesigner.platform.exception.DatabaseException;

/**
 * <B>Named Query Runner</B>
 * 
 * @author leesangboo
 * 
 */
public class NamedQueryRunner {

	private static final Log log = LogFactory.getLog(NamedQueryRunner.class);

	private volatile boolean pmdKnownBroken = false;

	/**
	 * NamedQueryRunner
	 * 
	 */
	public NamedQueryRunner() {
		super();
	}

	/**
	 * Constructor for QueryRunner, allows workaround for Oracle drivers
	 * 
	 * @param pmdKnownBroken
	 *            Oracle drivers don't support {@link ParameterMetaData#getParameterType(int) }; if <code>pmdKnownBroken</code> is set to true, we
	 *            won't even try it; if false, we'll try it, and if it breaks, we'll remember not to use it again.
	 */
	public NamedQueryRunner(boolean pmdKnownBroken) {
		super();
		this.pmdKnownBroken = pmdKnownBroken;
	}

	/**
	 * SELECT SQL을 실행한다.
	 * 
	 * @param conn
	 * @param sql
	 * @param pStmntValues
	 * @param processor
	 * @return
	 * @throws DatabaseException
	 */
	public Object query(Connection conn, String sql, ResultSetHandler handler, Map pStmntValues) throws SQLException {

		PreparedStatement pstmnt = null;
		ResultSet rs = null;

		try {

			// Build prepared statement
			pstmnt = this.buildPreparedStatement(new NamedParameterStatement(conn, sql), pStmntValues);
			//
			rs = pstmnt.executeQuery();
			Object results = handler.handle(rs);
			//
			return results;

		} catch (SQLException sqle) {
			DbUtils.close(rs);
			DbUtils.close(pstmnt);
			throw sqle;
		} finally {
			DbUtils.close(rs);
			DbUtils.close(pstmnt);
		}

	}

	/**
	 * INSERT, UPDATE, DELETE SQL을 실행한다.
	 * 
	 * @param conn
	 * @param sql
	 * @param pStmntValues
	 * @return
	 * @throws DatabaseException
	 */
	public int update(Connection conn, String sql, Map pStmntValues) throws SQLException {
		PreparedStatement pstmnt = null;
		try {
			// Build prepared statement
			pstmnt = this.buildPreparedStatement(new NamedParameterStatement(conn, sql), pStmntValues);
			// Attempt to exceute the statement
			int rows = pstmnt.executeUpdate();
			return rows;
		} catch (SQLException sqle) {
			DbUtils.close(pstmnt);
			throw sqle;
		} finally {
			DbUtils.close(pstmnt);
		}

	}

	/**
	 * 배치 업데이트를 실행한다.
	 * 
	 * @param conn
	 * @param sql
	 * @param pStmntValues
	 * @return
	 * @throws DatabaseException
	 * 
	 */
	public int[] batch(Connection conn, String sql, List pStmntValuesArray) throws SQLException {
		throw new SQLException("Does not support this NamedQueryRunner batch method.");
	}

	/**
	 * 
	 * PreparedStatement를 빌드한다.
	 * 
	 * @param nstmnt
	 * @param values
	 * @return
	 * @throws IllegalArgumentException
	 * @throws IllegalStateException
	 * @throws DatabaseException
	 */
	private PreparedStatement buildPreparedStatement(NamedParameterStatement nstmnt, Map values) throws IllegalArgumentException, IllegalStateException, DatabaseException {

		try {

			// If we have a null value here, then bail out.
			if (nstmnt == null) {
				throw new IllegalArgumentException("NamedParameterStatement is null.");
			}

			//
			boolean isPage = false;
			Map indexMap = nstmnt.getIndexMap();
			Object[] indexKeys = indexMap.keySet().toArray();
			ParameterMetaData pmd = nstmnt.getParameterMetaData();

			// 페이징 모드 여부 확인
			if (values != null && !values.isEmpty()) {
				Object[] valuesKeys = values.keySet().toArray();
				if (valuesKeys.length == 2 && values.containsKey(Page.PAGE_START_PARAM_KEY) && values.containsKey(Page.PAGE_END_PARAM_KEY)
						&& nstmnt.getParsedQuery().indexOf(PagingSQLHandler.PAGING_ALIAS) > -1) {
					isPage = true;
				}
			}

			// 1. SQL내에 바인드 변수가 존재할 경우
			if (indexKeys != null && indexKeys.length > 0) {

				for (int i = 0; i < indexKeys.length; i++) {
					String indexKey = (String) indexKeys[i];

					// 1-1.파라메터속에 키가 존재할 경우
					if (values != null && values.containsKey(indexKey) && values.get(indexKey) != null) {
						Object value = values.get(indexKeys[i]);
						nstmnt.setObject(indexKey, value);
						if (log.isDebugEnabled()) {
							log.debug("setObject : indexKey=" + indexKey + ", value=" + value);
						}

						// 1-2.파라메터속에 키가 존재하지 않을 경우
					} else {
						// VARCHAR works with many drivers regardless
						// of the actual column type. Oddly, NULL and
						// OTHER don't work with Oracle's drivers.
						int sqlType = Types.VARCHAR;
						if (!pmdKnownBroken) {
							try {
								sqlType = pmd.getParameterType(i + 1);
							} catch (SQLException e) {
								pmdKnownBroken = true;
							}
						}
						// TODO: 데이터 베이스 NULL 값 입력 처리
						nstmnt.setNull(indexKey, sqlType);
						// nstmnt.setString(indexKey, "");

						if (log.isDebugEnabled()) {
							log.debug("setNull : indexKey=" + indexKey + ", sqlType=" + sqlType);
						}
					}

				}
			}

			// 2. 바인드 값 맵이 NULL이고 페이징 모드일 경우
			if (isPage && (values == null || values.isEmpty())) {
				nstmnt.setObject(Page.PAGE_START_PARAM_KEY, values.get(Page.PAGE_START_PARAM_KEY));
				nstmnt.setObject(Page.PAGE_END_PARAM_KEY, values.get(Page.PAGE_END_PARAM_KEY));
			}

			return nstmnt.getStatement();

		} catch (SQLException sqle) {
			String msg = "The prepared statement could not be formed with the given values : " + sqle.getMessage();
			log.error(msg, sqle);
			throw new DatabaseException(msg, sqle);
		} catch (Exception ex) {
			String msg = "Error has occurred while trying to build a prepared statement : " + ex.getMessage();
			log.error(msg, ex);
			throw new DatabaseException(msg, ex);
		}
	}

}
