/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.platform.db.table.sql;

import java.util.List;

import org.jdesigner.platform.db.table.model.Column;

/**
 * SQLGeneratorHelper
 * 
 * @author leesangboo
 * 
 */
public class SQLGeneratorHelper {

	/**
	 * 컬럼명을 해석하여, 컴머로 구분된 칼럼명으로 반환한다.
	 * 
	 * @param columns
	 *            - 테이블 컬럼 리스트
	 * @return 컴마로 구분된 컬럼명
	 */
	public String parseColumnNames(List columns) {
		StringBuffer buffer = new StringBuffer();
		int size = columns.size();
		for (int i = 0; i < size; i++) {
			Column column = (Column) columns.get(i);
			buffer.append("     ").append(column.getColumnName());
			if (i != (size - 1)) {
				buffer.append(", \n");
			}
		}
		return buffer.toString();
	}

	/**
	 * 컬럼명을 해석하여, 컴머로 구분된 칼럼명으로 반환한다.
	 * 
	 * @param columns
	 *            - 테이블 컬럼 리스트
	 * @return 컴마로 구분된 컬럼명
	 */
	public String parseColumnNames(List columns, String[] processTargetColumns) {
		if (processTargetColumns == null || processTargetColumns.length < 1) {
			return parseColumnNames(columns);
		}

		StringBuffer buffer = new StringBuffer();
		int size = processTargetColumns.length;
		for (int i = 0; i < size; i++) {
			String columnName = processTargetColumns[i];
			if (hasColumnName(columns, columnName)) {
				buffer.append("     ").append(columnName).append(", \n");
			}
		}

		return removeLastComma(buffer);
	}

	/**
	 * 조회,수정,삭제 SQL에 필요한 WHERE구문을 반환한다.
	 * 
	 * @param primaryKeys
	 *            - 프라이머리키 리스트
	 * @return SQL WHERE 구문
	 */
	public String makeWhereByPrimaryKeys(List primaryKeys) {
		StringBuffer buffer = new StringBuffer();
		int size = primaryKeys.size();
		for (int i = 0; i < size; i++) {
			buffer.append(primaryKeys.get(i).toString().toUpperCase()).append(" = :" + primaryKeys.get(i).toString().toLowerCase() + " \n");
			if (i != (size - 1)) {
				buffer.append("        AND ");
			}
		}
		return buffer.toString();
	}

	/**
	 * 입력 SQL의 바인딩 쿼리 구문을 반환한다.
	 * 
	 * @param columns
	 *            - 테이블 컬럼 리스트
	 * @return
	 * @throws Exception
	 */
	public String makeInsertableBinding(List columns) {
		StringBuffer buffer = new StringBuffer();
		int size = columns.size();
		for (int i = 0; i < size; i++) {
			Column column = (Column) columns.get(i);
			buffer.append("     ").append(":").append(column.getColumnName().toLowerCase());
			if (i != (size - 1)) {
				buffer.append(", \n");
			}
		}
		return buffer.toString();
	}

	/**
	 * 
	 * @param columns
	 * @param processTargetColumns
	 * @return
	 */
	public String makeInsertableBinding(List columns, String[] processTargetColumns) {
		if (processTargetColumns == null || processTargetColumns.length < 1) {
			return makeInsertableBinding(columns);
		}

		StringBuffer buffer = new StringBuffer();
		int size = processTargetColumns.length;
		for (int i = 0; i < size; i++) {
			String columnName = processTargetColumns[i];
			if (hasColumnName(columns, columnName)) {
				buffer.append("     ").append(":").append(columnName.toLowerCase()).append(", \n");
			}
		}
		return removeLastComma(buffer);
	}

	/**
	 * 수정 SQL의 바인딩 쿼리 구문을 반환한다.
	 * 
	 * @param columns
	 *            - 테이블 컬럼 리스트
	 * @return
	 * @throws Exception
	 */
	public String makeUpdatableBinding(List columns) {
		StringBuffer buffer = new StringBuffer();
		int size = columns.size();
		for (int i = 0; i < size; i++) {
			Column column = (Column) columns.get(i);
			buffer.append("     ").append(column.getColumnName()).append(" = ").append(":").append(column.getColumnName().toLowerCase());
			if (i != (size - 1)) {
				buffer.append(", \n");
			}
		}
		return buffer.toString();
	}

	/**
	 * 수정 SQL의 바인딩 쿼리 구문을 반환한다.
	 * 
	 * @param columns
	 * @param processTargetColumns
	 * @return
	 */
	public String makeUpdatableBinding(List columns, String[] processTargetColumns) {
		if (processTargetColumns == null || processTargetColumns.length < 1) {
			return makeUpdatableBinding(columns);
		}

		StringBuffer buffer = new StringBuffer();
		int size = processTargetColumns.length;
		for (int i = 0; i < size; i++) {
			String columnName = processTargetColumns[i];
			if (hasColumnName(columns, columnName)) {
				buffer.append("     ").append(columnName).append(" = ").append(":").append(columnName.toLowerCase()).append(", \n");
			}
		}
		return removeLastComma(buffer);
	}

	/**
	 * 컬럼 리스트 속에 컬럼명이 존재하는지 확인한다.
	 * 
	 * @param columns
	 * @param columnName
	 * @return
	 */
	public boolean hasColumnName(List columns, String columnName) {
		if (columns == null) {
			return false;
		}
		int size = columns.size();
		for (int i = 0; i < size; i++) {
			Column column = (Column) columns.get(i);
			// System.out.println("column.getColumnName()=" +
			// column.getColumnName());
			// System.out.println("columnName=" + columnName);
			if (column.getColumnName().toLowerCase().equals(columnName)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * 마지막의 ,를 제거한 문자열을 반환한다.
	 * 
	 * @param buffer
	 * @return
	 */
	public String removeLastComma(StringBuffer buffer) {
		return org.apache.commons.lang.StringUtils.substringBeforeLast(buffer.toString(), ",") + org.apache.commons.lang.StringUtils.substringAfterLast(buffer.toString(), ",");
	}
}
