/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.platform.scheduler;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Timer;

import javax.servlet.ServletContext;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jdesigner.platform.scheduler.task.SchedulerTask;
import org.jdesigner.platform.scheduler.task.SchedulerTaskInfo;
import org.jdesigner.platform.util.StringUtils;

/**
 * 
 * <B> SchedulerLifeCycle</B>
 * 
 * @author SangBoo Lee
 * @since 1.0
 * 
 */
public class SchedulerFactory {

	private static final Log log = LogFactory.getLog(SchedulerFactory.class);

	private static class SchedulerFactoryHolder {
		static SchedulerFactory instance = new SchedulerFactory();
	}

	public static SchedulerFactory getInstance() {
		return SchedulerFactoryHolder.instance;
	}

	private List timerList = new ArrayList();

	/**
	 * Creates and instantiate a <code>SchedulerTask</code> based on the provided <code>SchedulerTask</code> object.
	 * 
	 * @param taskInfo
	 *            a <code>SchedulerTaskInfo</code> object.
	 * @return a <code>SchedulerTask</code> object.
	 * 
	 */
	public SchedulerTask createSchedulerTask(ServletContext context, SchedulerTaskInfo taskInfo) {

		try {

			Class taskObject = Class.forName(taskInfo.getTask());
			SchedulerTask task = (SchedulerTask) taskObject.newInstance();
			task.setInitParameters(taskInfo.getParameters());
			task.setServletContext(context);

			log.debug("Scheduler task created : info=[" + ToStringBuilder.reflectionToString(taskInfo) + "]");

			return task;

		} catch (java.lang.ClassNotFoundException e) {
			throw new SchedulerException("Provided class name does not exists. " + "Exception is: " + e);
		} catch (java.lang.IllegalAccessException e) {
			throw new SchedulerException("Cannot instantiate provided class name. " + "Exception is: " + e);
		} catch (java.lang.InstantiationException e) {
			throw new SchedulerException("Cannot instantiate provided class name. " + "Exception is: " + e);
		}
	}

	/**
	 * Launches a <code>Timer</code> object with the provided task information.
	 * 
	 * @param task
	 *            a <code>SchedulerTask</code> object.
	 * @param taskInfo
	 *            a <code>SchedulerTaskInfo</code> object.
	 * 
	 */
	public void launchTimer(SchedulerTask task, SchedulerTaskInfo taskInfo) {

		Timer timer = new Timer(true);
		Date time = null;

		/*
		 * Parse the date from taskInfo if provided.
		 */
		if (StringUtils.isNotEmpty(taskInfo.getTime())) {

			// Create the date formert
			DateFormat formatter = null;

			/*
			 * Check for which format was provide yyyy-MM-dd HH:mm:ss, HH:mm:ss or yyy-MM-dd
			 */
			if (taskInfo.getTime().indexOf("-") > 0 && taskInfo.getTime().indexOf(":") > 0) {

				// Full Date yyyy-MM-dd HH:mm:ss
				formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			} else if (taskInfo.getTime().indexOf("-") > 0 && taskInfo.getTime().indexOf(":") < 0) {

				// Just the date yyyy-MM-dd
				formatter = new SimpleDateFormat("yyyy-MM-dd");
			} else if (taskInfo.getTime().indexOf("-") < 0 && taskInfo.getTime().indexOf(":") > 0) {

				// Just a time HH:mm:ss
				formatter = new SimpleDateFormat("HH:mm:ss");
			}

			try {
				time = formatter.parse(taskInfo.getTime());
			} catch (java.text.ParseException e) {
				throw new IllegalArgumentException("Could not parse date provided date: '" + time + "'. " + "Exception is: " + e);
			}

			// In case of just time, Add the appropriate date
			if (taskInfo.getTime().indexOf("-") < 0 && taskInfo.getTime().indexOf(":") > 0) {

				// Get the User provided Time
				Calendar userCal = Calendar.getInstance();
				userCal.setTime(time);

				// Get System Calendar Date
				Calendar sys = Calendar.getInstance();

				// Set the date for the time provided
				userCal.set(Calendar.YEAR, sys.get(java.util.Calendar.YEAR));
				userCal.set(Calendar.MONTH, sys.get(java.util.Calendar.MONTH));
				userCal.set(Calendar.DAY_OF_MONTH, sys.get(Calendar.DAY_OF_MONTH));

				// Compare the two dates.
				if (userCal.getTime().getTime() < sys.getTime().getTime()) {

					// Time has passed. Add one day
					userCal.add(Calendar.DAY_OF_MONTH, 1);
				}

				// Set the time object
				time = userCal.getTime();
			}
		}

		/*
		 * Execute the proper Timer method call based on arguments in taskInfo If not method excactly match the number and type of supplied arguments,
		 * we throw an error.
		 */
		if (time != null && StringUtils.isEmpty(taskInfo.getPeriod()) && StringUtils.isEmpty(taskInfo.getDelay())) {
			timer.schedule(task, time);
		} else if (time != null && StringUtils.isNotEmpty(taskInfo.getPeriod()) && StringUtils.isEmpty(taskInfo.getDelay())) {
			timer.schedule(task, time, SchedulerUtil.parseTime(taskInfo.getPeriod()));
		} else if (time == null && StringUtils.isEmpty(taskInfo.getPeriod()) && StringUtils.isNotEmpty(taskInfo.getDelay())) {
			timer.schedule(task, SchedulerUtil.parseTime(taskInfo.getDelay()));

		} else if (time == null && StringUtils.isNotEmpty(taskInfo.getPeriod()) && StringUtils.isNotEmpty(taskInfo.getDelay())) {
			timer.schedule(task, SchedulerUtil.parseTime(taskInfo.getDelay()), SchedulerUtil.parseTime(taskInfo.getPeriod()));
		} else {
			throw new SchedulerException("No Timer.shedule() method could be found matching " + "supplied arguments. ");
		}

		//
		timerList.add(timer);
	}

	public List getTimerList() {
		return timerList;
	}

	public void setTimerList(List timerList) {
		this.timerList = timerList;
	}

}
