package org.jdesigner.platform.util;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.Closeable;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import net.sf.jazzlib.Deflater;
import net.sf.jazzlib.ZipEntry;
import net.sf.jazzlib.ZipInputStream;
import net.sf.jazzlib.ZipOutputStream;

/**
 * ZipUtils
 * 
 * @author leesangboo
 * 
 */
public class ZipUtils {

	private static final int BATCH_SIZE = 4096;

	private static long copyLarge(InputStream input, OutputStream output) throws IOException {
		byte buffer[] = new byte[BATCH_SIZE];
		long count = 0L;
		for (int n = 0; -1 != (n = input.read(buffer));) {
			output.write(buffer, 0, n);
			count += n;
		}

		return count;
	}

	private static final void closeQuietly(Closeable zos) {
		try {
			if (zos != null) {
				zos.close();
			}
		} catch (Exception e) {
		}
	}

	private static String normalizeEntryName(String entryName) {
		return entryName.replace(File.separatorChar, '/');
	}

	private static String getEntryName(File rootDir, File zippedFile) {
		if (rootDir != null && zippedFile.getPath().startsWith(rootDir.getPath())) {
			return zippedFile.getPath().substring(rootDir.getPath().length() + 1);
		} else {
			return zippedFile.getPath();
		}
	}

	private static void zipFile(File zippedFile, String entryName, ZipOutputStream zos) throws FileNotFoundException, IOException {
		FileInputStream fis = new FileInputStream(zippedFile);
		ZipEntry anEntry = new ZipEntry(normalizeEntryName(entryName));
		zos.putNextEntry(anEntry);
		copyLarge(fis, zos);
		fis.close();
		zos.flush();
	}

	private static void zipDir(File zippedDir, String entryName, ZipOutputStream zos) throws FileNotFoundException, IOException {
		File[] dirList = zippedDir.listFiles();
		for (int i = 0; i < dirList.length; i++) {
			File f = dirList[i];
			if (f.isDirectory()) {
				zipDir(f, entryName + "/" + f.getName(), zos);
				continue;
			}
			zipFile(f, entryName + "/" + f.getName(), zos);
		}
	}

	public static final void zipFile(File rootDir, File zippedFile, File zipFile, int level) throws FileNotFoundException, IOException {
		ZipOutputStream zos = null;
		try {
			zos = new ZipOutputStream(new BufferedOutputStream(new FileOutputStream(zipFile)));
			zos.setLevel(level);
			zipFile(zippedFile, getEntryName(rootDir, zippedFile), zos);
		} finally {
			closeQuietly(zos);
		}
	}

	public static final void zipDir(File rootDir, File zippedDir, File zipFile, int level) throws FileNotFoundException, IOException {
		ZipOutputStream zos = null;
		try {
			zos = new ZipOutputStream(new BufferedOutputStream(new FileOutputStream(zipFile)));
			zos.setLevel(level);
			zipDir(zippedDir, getEntryName(rootDir, zippedDir), zos);
		} finally {
			closeQuietly(zos);
		}
	}

	public static final void zip(File rootDir, List zipped, File zipFile, int level) throws FileNotFoundException, IOException {
		ZipOutputStream zos = null;
		try {
			zos = new ZipOutputStream(new BufferedOutputStream(new FileOutputStream(zipFile)));
			zos.setLevel(level);
			for (int i = 0; i < zipped.size(); i++) {
				File f = (File) zipped.get(i);
				if (f.isDirectory()) {
					zipDir(f, getEntryName(rootDir, f), zos);
				} else {
					zipFile(f, getEntryName(rootDir, f), zos);
				}
			}
		} finally {
			closeQuietly(zos);
		}
	}

	public static final void unzip(File zipFile, File outDir) throws FileNotFoundException, IOException {
		ZipInputStream in = null;
		try {
			in = new ZipInputStream(new BufferedInputStream(new FileInputStream(zipFile)));
			ZipEntry entry;
			while ((entry = in.getNextEntry()) != null) {
				File file = new File(outDir, entry.getName());
				// System.out.println(file);
				if (!file.exists()) {
					if (!file.getParentFile().exists() && !file.getParentFile().mkdirs()) {
						throw new IOException("Could not create directory : " + file.getParentFile());
					}
					if (!file.createNewFile()) {
						throw new IOException("Could not create file : " + file.getParentFile());
					}
				}
				BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(file));
				copyLarge(in, out);
				out.flush();
				out.close();
			}
		} finally {
			closeQuietly(in);
		}
	}

	public static final List unzip(File zipFile) throws FileNotFoundException, IOException {
		List list = new ArrayList(0);
		ZipInputStream in = null;
		try {
			in = new ZipInputStream(new BufferedInputStream(new FileInputStream(zipFile)));
			ZipEntry entry;
			while ((entry = in.getNextEntry()) != null) {
				list.add(entry.getName());
			}
			return list;
		} finally {
			closeQuietly(in);
		}
	}

	public static void test() throws Exception {

		// 압축하기
		List sourceFiles = new ArrayList();
		sourceFiles.add(new File("c:/temp/1.gif")); //
		sourceFiles.add(new File("c:/temp/2.gif")); //
		sourceFiles.add(new File("c:/temp/한글.gif")); //

		File rootDir = new File("c:/temp/");
		File zipFile = new File(rootDir, "download.zip");
		ZipUtils.zip(rootDir, sourceFiles, zipFile, Deflater.DEFAULT_COMPRESSION);

		// 압축 풀기
		List result = ZipUtils.unzip(zipFile);
		for (int i = 0; i < result.size(); i++) {
			System.out.println(result.get(i));
		}

	}
}
