/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.platform.web.multipart;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jdesigner.platform.context.ApplicationConfig;
import org.jdesigner.platform.context.ApplicationContextFactory;
import org.jdesigner.platform.util.FileUtils;
import org.jdesigner.platform.web.WebConstants;

/**
 * MultipartRequestWrapper
 * 
 * @author leesangboo
 * 
 */
public class MultipartRequestWrapper extends HttpServletRequestWrapper {
	private static final Log log = LogFactory.getLog(MultipartRequestWrapper.class);

	private String PARAM_ENCODING; // 파라메터 인코딩

	private String REPOGITORY_SAVE_DIR; // 저장 디렉토리

	private long MAX_FILE_SIZE; // 업로드 가능한 최고 파일 크기

	private String LIMIT_FILE_SUFFIX; // 업로드 제한 파일 확장자

	private Map parameters;

	private Map files;

	private List formFileList;

	/**
	 * 멀티파드 요청 레퍼 생성자
	 * 
	 * @param request
	 *            HTTP 요청
	 * @throws Exception
	 *             파일 업로드 에러 및 기타
	 */
	public MultipartRequestWrapper(HttpServletRequest request) {
		super(request);
		this.initialize();
		this.fileUploadProcess(request);
	}

	/**
	 * 파입업로드를 위한 초기화 작업
	 * 
	 */
	private void initialize() {
		try {
			//
			ApplicationContextFactory appFactory = ApplicationContextFactory.getInstance();
			ApplicationConfig appConfig = appFactory.getApplicationContext().getApplicationConfig();

			//
			PARAM_ENCODING = appConfig.getProperty("platform.web.parameter.encoding");
			REPOGITORY_SAVE_DIR = appConfig.getProperty("platform.web.multipart.repositorySaveDir"); // 저장
			// 디렉토리
			MAX_FILE_SIZE = Long.parseLong(appConfig.getProperty("platform.web.multipart.maxsize")); // 업로드가능한
			// 최고
			// 파일
			// 크기
			// (10M)
			LIMIT_FILE_SUFFIX = appConfig.getProperty("platform.web.multipart.limitFileSuffix"); // 업로드
			// 제한
			// 파일
			// 확장자

		} catch (Exception ex) {
			log.error(ex);
			MultipartException mex = new MultipartException(MultipartException.INITIALIZE_ERROR, "Multipart properties initialize exception.", ex);
			this.setAttribute(WebConstants.MULTIPART_ERROR_KEY, mex);
			// throw mex;
		}
	}

	/**
	 * 파일업로드 처리
	 * 
	 * @param request
	 * @param saveDir
	 * @throws Exception
	 */
	private void fileUploadProcess(HttpServletRequest request) {

		parameters = new HashMap();
		files = new HashMap();
		formFileList = new ArrayList();

		try {

			log.debug("Multipart file upload request.");

			request.setCharacterEncoding(PARAM_ENCODING); //

			DiskFileItemFactory factory = new DiskFileItemFactory();
			factory.setSizeThreshold(0x3200000);
			String tempDir = System.getProperty("java.io.tmpdir");
			;
			factory.setRepository(new File(tempDir));
			ServletFileUpload upload = new ServletFileUpload(factory);
			upload.setSizeMax(MAX_FILE_SIZE);

			List fileList = null;
			try {
				fileList = upload.parseRequest(request);
			} catch (ServletFileUpload.SizeLimitExceededException ex) {
				String msg = "File size has been exceeded : [" + FileUtils.getDisplayFileSize(MAX_FILE_SIZE) + "]";
				throw new MultipartException(MultipartException.MAX_FILE_SIZE_ERROR, FileUtils.getDisplayFileSize(MAX_FILE_SIZE), msg);
			}

			for (Enumeration en = request.getParameterNames(); en.hasMoreElements();) {
				String name = (String) en.nextElement();
				String values[] = request.getParameterValues(name);
				String encodedValues[] = null;
				if (values != null && values.length > 0) {
					encodedValues = new String[values.length];
					for (int x = 0; x < values.length; x++) {
						encodedValues[x] = isoToEncoding(values[x], PARAM_ENCODING);// /////
					}

					parameters.put(name, encodedValues);
				}
			}

			int fCnt = 0;
			List paramFileNames = new ArrayList();
			for (Iterator i = fileList.iterator(); i.hasNext();) {
				FileItem item = (FileItem) i.next();
				if (item.isFormField()) {
					List values;
					if (parameters.get(item.getFieldName()) != null) {
						values = (List) parameters.get(item.getFieldName());
					} else {
						values = new ArrayList();
					}
					values.add(isoToEncoding(item.getString(), PARAM_ENCODING)); // /////
					parameters.put(item.getFieldName(), values);

				} else if (item.getSize() != 0L) {
					String fullFileName = item.getName();
					String subFileName = fullFileName.substring(fullFileName.lastIndexOf("\\") + 1);
					if (LIMIT_FILE_SUFFIX != null && !LIMIT_FILE_SUFFIX.equals("")) {
						String limitFileSuffix[] = LIMIT_FILE_SUFFIX.split(",");
						for (int z = 0; z < limitFileSuffix.length; z++) {
							if (fullFileName.endsWith(limitFileSuffix[z])) {
								String msg = "Limited file extension of uploaded file : [" + limitFileSuffix[z] + "]";
								throw new MultipartException(MultipartException.LIMIT_FILE_SUFFIX_ERROR, limitFileSuffix[z], msg);
							}
						}
					}

					List values = null;
					if (files.get(item.getFieldName()) != null) {
						values = (List) files.get(item.getFieldName());
					} else {
						values = new ArrayList();
					}

					// String webAppPath =
					// request.getSession().getServletContext().getRealPath("/");
					String saveDir = REPOGITORY_SAVE_DIR;
					//
					String savedFileName = saveFile(saveDir, subFileName, item.get());
					log.debug("Form file field : name=[" + item.getFieldName() + "], savedFilePath=[" + saveDir + savedFileName + "] File Saved.");

					paramFileNames.add(saveDir + savedFileName); // 파라메터값으로 파일의
					// 전체경로 값
					// 업데이트
					parameters.put(item.getFieldName(), paramFileNames);
					formFileList.add(item.getFieldName());
					files.put(item.getFieldName(), values);
					fCnt++;

				}
			}

			log.debug("Multipart uploaded files count : [" + fCnt + "]");

		} catch (MultipartException mex) {
			log.error(mex.getMessage(), mex);
			this.setAttribute(WebConstants.MULTIPART_ERROR_KEY, mex);
			reset();
			// throw mex;
		} catch (Exception ex) {
			String msg = "Multipart File Upload Error : " + ex.getMessage();
			log.error(msg, ex);
			MultipartException mex = new MultipartException(MultipartException.UN_CATCHED_ERROR, msg, ex);
			;
			this.setAttribute(WebConstants.MULTIPART_ERROR_KEY, mex);
			reset();
			// throw mex;
		}
	}

	/*
	 * (비Javadoc)
	 * 
	 * @see javax.servlet.ServletRequest#getParameter(java.lang.String)
	 */
	public String getParameter(String name) {
		List v = (List) parameters.get(name);
		if (v != null && v.size() > 0) {
			return (String) v.get(0);
		} else {
			return null;
		}
	}

	/*
	 * (비Javadoc)
	 * 
	 * @see javax.servlet.ServletRequest#getParameterNames()
	 */
	public Enumeration getParameterNames() {
		return Collections.enumeration(parameters.keySet());
	}

	/*
	 * (비Javadoc)
	 * 
	 * @see javax.servlet.ServletRequest#getParameterValues(java.lang.String)
	 */
	public String[] getParameterValues(String name) {
		List v = (List) parameters.get(name);
		String values[] = new String[v.size()];
		if (v != null && v.size() > 0) {
			for (int i = 0; i < v.size(); i++) {
				values[i] = (String) v.get(i);
			}

		}
		return values;
	}

	/*
	 * (비Javadoc)
	 * 
	 * @see javax.servlet.ServletRequest#getParameterMap()
	 */
	public Map getParameterMap() {
		Map map = new HashMap();
		String names[] = parseEnumParamNames();
		for (int i = 0; i < names.length; i++) {
			map.put(names[i], getParameterValues(names[i]));
		}

		return map;
	}

	/**
	 * 파라메터명을 반환한다.
	 * 
	 * @return 파라메터명 배열
	 */
	public String[] parseEnumParamNames() {
		List paramNameList = new ArrayList();
		for (Enumeration paramNames = getParameterNames(); paramNames.hasMoreElements(); paramNameList.add(paramNames.nextElement())) {
			;
		}
		String returnParamNames[] = new String[paramNameList.size()];
		for (int i = 0; i < paramNameList.size(); i++) {
			returnParamNames[i] = (String) paramNameList.get(i);
		}

		return returnParamNames;
	}

	/**
	 * 
	 * @return
	 */
	public String[] getFormFileNames() {
		Object[] fieldNams = formFileList.toArray();
		if (fieldNams == null || fieldNams.length < 1) {
			fieldNams = new String[0];
		}
		int size = fieldNams.length;
		String[] formFileNames = new String[size];
		for (int i = 0; i < size; i++) {
			formFileNames[i] = (String) fieldNams[i];
		}
		return formFileNames;
	}

	/**
	 * 입력받은 문자의 인코딩을 변경 후, 반환한다.
	 * 
	 * @param str
	 *            입력 문자열
	 * @param encoding
	 *            변경할 인코딩
	 * @return 인코딩된 문자열
	 * @throws Exception
	 *             지원되지않는 인코딩 예외
	 */
	private String isoToEncoding(String str, String encoding) throws Exception {
		String returnStr = new String(str.getBytes("8859_1"), encoding);
		if (returnStr != null && !returnStr.equals("")) {
			return returnStr;
		} else {
			return "";
		}
	}

	/**
	 * 파일을 저장한다.
	 * 
	 * @param dir
	 *            저장할 디렉토리
	 * @param fileName
	 *            파일명
	 * @param content
	 *            컨텐츠
	 * @return 업로드된 파일명(디렉토리포함 ex)c:/file/upload/myfile.zip)을 반환한다.
	 * @throws Exception
	 */
	private String saveFile(String dir, String fileName, byte content[]) throws Exception {
		String savedFileName;
		savedFileName = null;
		FileOutputStream out = null;

		try {
			File saveDir = new File(dir);
			if (!saveDir.isDirectory()) {
				saveDir.mkdir();
			}

			File writeFile = renameFile(dir, fileName);
			out = new FileOutputStream(writeFile);
			out.write(content);
			out.flush();
			out.close();

			savedFileName = writeFile.getName();
		} catch (Exception ex) {
			log.error("File save failed : " + ex.getMessage());
			out.close();
		} finally {
			out.close();
		}
		return savedFileName;
	}

	/**
	 * 업로드할 디렉토리내에 중복되는 파일이 존재할 경우, 파일명을 변경하여 저장 한다.
	 * 
	 * @param dir
	 *            업로드 디렉토리
	 * @param fileName
	 *            파일명
	 * @return 제 정의된 파일
	 * @throws Exception
	 */
	private File renameFile(String dir, String fileName) throws Exception {
		File sourceFile = new File(dir, fileName);
		File targetFile = null;
		if (sourceFile.exists()) {
			for (int count = 1; count < 1000; count++) {
				String body = null;
				String ext = null;
				int dot = fileName.lastIndexOf(".");
				if (dot != -1) {
					body = fileName.substring(0, dot);
					ext = fileName.substring(dot);
				} else {
					body = fileName;
					ext = "";
				}
				String newFileName = body + "_" + count + ext;
				targetFile = new File(dir, newFileName);
				if (!targetFile.exists()) {
					break;
				}
			}

		} else {
			targetFile = sourceFile;
		}
		return targetFile;
	}

	/**
	 * 변수를 리셋한다.
	 */
	private void reset() {
		// parameters = null;
		// files = null;
		// formFileList = null;
	}

}
