/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.platform.web.tiles;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.tiles.DefinitionsFactoryException;
import org.apache.struts.tiles.FactoryNotFoundException;
import org.apache.struts.tiles.xmlDefinition.DefinitionsFactory;
import org.apache.struts.tiles.xmlDefinition.FactorySet;
import org.apache.struts.tiles.xmlDefinition.XmlDefinitionsSet;
import org.apache.struts.tiles.xmlDefinition.XmlParser;
import org.xml.sax.SAXException;

import org.jdesigner.platform.context.ApplicationContextFactory;
import org.jdesigner.platform.util.StringUtils;
import org.jdesigner.platform.web.XMLFileReader;

/**
 * Component Definitions factory. This implementation of Component Definitions factory allows i18n Component definitions factory allowing i18n
 * definition. A definition is retrieved by its name, and using locale setted in appropriate session context. Definitions are defined in different
 * files, one for each locale. A definition file is loaded using common name extended with locale code (ex : templateDefinitions_fr.xml). If no file
 * is found under this name, use default file.
 */
public class ChannelFactorySet extends FactorySet {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private static final Log log = LogFactory.getLog(ChannelFactorySet.class);

	/**
	 * Debug flag.
	 * 
	 * @deprecated This will be removed in a release after Struts 1.2.
	 */
	public static final boolean debug = false;

	/**
	 * Default name.
	 */
	public static final String DEFAULT_DEFINITIONS_FILE_NAME = "/WEB-INF/templateDefinitions.xml";

	/**
	 * Config file parameter name.
	 */
	public static final String DEFINITIONS_CONFIG_PARAMETER_NAME = "definitions-config";

	/**
	 * Config file parameter name.
	 */
	public static final String PARSER_DETAILS_PARAMETER_NAME = "definitions-parser-details";

	/**
	 * Config file parameter name.
	 */
	public static final String PARSER_VALIDATE_PARAMETER_NAME = "definitions-parser-validate";

	/**
	 * Config file parameter name.
	 */
	public static final String FACTORY_SELECTOR_KEY = "tiles.ChannelFactorySet.factorySelectorKey";

	/**
	 * Default filenames extension.
	 */
	public static final String FILENAME_EXTENSION = ".xml";

	/**
	 * Default factory.
	 */
	protected DefinitionsFactory defaultFactory = null;

	/**
	 * Xml parser used.
	 */
	protected XmlParser xmlParser = null;

	/**
	 * Names of default file containing definition descriptions.
	 */
	private List filenames;

	/**
	 * Collection of already loaded definitions set, referenced by their suffix.
	 */
	private Map loaded = null;

	/**
	 * Do we want validating parser. Default is <code>false</code>. Can be set from servlet config file.
	 */
	protected boolean isValidatingParser = false;

	/**
	 * Parser detail level. Default is 0. Can be set from servlet config file.
	 */
	protected int parserDetailLevel = 0;

	/**
	 * Parameterless Constructor. Method initFactory must be called prior to any use of created factory.
	 */
	public ChannelFactorySet() {
		super();
	}

	/**
	 * Constructor. Init the factory by reading appropriate configuration file.
	 * 
	 * @throws FactoryNotFoundException
	 *             Can't find factory configuration file.
	 */
	public ChannelFactorySet(ServletContext servletContext, Map properties) throws DefinitionsFactoryException {

		initFactory(servletContext, properties);
	}

	/**
	 * Initialization method. Init the factory by reading appropriate configuration file. This method is called exactly once immediately after factory
	 * creation in case of internal creation (by DefinitionUtil).
	 * 
	 * @param servletContext
	 *            Servlet Context passed to newly created factory.
	 * @param properties
	 *            Map of name/property passed to newly created factory. Map can contains more properties than requested.
	 * @throws DefinitionsFactoryException
	 *             An error occur during initialization.
	 */
	public void initFactory(ServletContext servletContext, Map properties) throws DefinitionsFactoryException {

		// Set some property values
		String value = (String) properties.get(PARSER_VALIDATE_PARAMETER_NAME);
		if (value != null) {
			isValidatingParser = Boolean.valueOf(value).booleanValue();
		}

		value = (String) properties.get(PARSER_DETAILS_PARAMETER_NAME);
		if (value != null) {
			try {
				parserDetailLevel = Integer.valueOf(value).intValue();

			} catch (NumberFormatException ex) {
				log.error("Bad format for parameter '" + PARSER_DETAILS_PARAMETER_NAME + "'. Integer expected.");
			}
		}

		String filename = null;

		log.info("Loading tiles configuration files using pattern matching, use the option.");

		ApplicationContextFactory factory = ApplicationContextFactory.getInstance();
		String appPath = factory.getApplicationContext().getAppPath();

		String config = (String) properties.get(DEFINITIONS_CONFIG_PARAMETER_NAME);
		if (config == null || config.split(",").length < 2) {
			throw new DefinitionsFactoryException("Not setting parameter[" + DEFINITIONS_CONFIG_PARAMETER_NAME + "] values : ex)/WEB-INF/config/,*-tiles-defs.xml");
		}
		String configXmlBasePath = config.split(",")[0];
		String configXmlPattern = config.split(",")[1];

		if (StringUtils.isEmpty(configXmlBasePath) || StringUtils.isEmpty(configXmlPattern)) {
			String msg = "Tiles plugin initialization parameters [definitions-config-base-path] or [definitions-config-pattern], please set up.";
			log.error(msg);
			throw new DefinitionsFactoryException(msg);
		}

		String[] pathArray = null;
		try {
			pathArray = XMLFileReader.getAllFilePathArray(appPath, configXmlBasePath, configXmlPattern);
		} catch (Exception e) {
			String msg = "XMLFileReader exception : " + e.getMessage();
			log.error(msg);
			throw new DefinitionsFactoryException(msg);
		}

		if (pathArray != null && pathArray.length > 0) {
			String commaCombinedPath = "";
			for (int i = 0, size = pathArray.length; i < size; i++) {
				commaCombinedPath += "," + pathArray[i];
			}
			filename = commaCombinedPath.substring(1);
			log.info("Loaded tiles configuration files : [" + filename + "]");
		} else {
			String msg = "Not found tiles configuration files.";
			log.error(msg);
			throw new DefinitionsFactoryException(msg);
		}

		if (filename != null) { // Use provided filename
			try {
				initFactory(servletContext, filename);
				if (log.isDebugEnabled()) {
					log.debug("Factory initialized from file '" + filename + "'.");
				}
			} catch (FileNotFoundException ex) { // A filename is specified,
				// throw appropriate error.
				log.error(ex.getMessage() + " : Can't find file '" + filename + "'");
				throw new FactoryNotFoundException(ex.getMessage() + " : Can't find file '" + filename + "'");
			}

		} else {
			// try each default file names
			throw new DefinitionsFactoryException("Not found tiles defination files.");
		}

		log.info("Initialized tiles channel factory set.");

	}

	/**
	 * Initialization method. Init the factory by reading appropriate configuration file. This method is called exactly once immediately after factory
	 * creation in case of internal creation (by DefinitionUtil).
	 * 
	 * @param servletContext
	 *            Servlet Context passed to newly created factory.
	 * @param proposedFilename
	 *            File names, comma separated, to use as base file names.
	 * @throws DefinitionsFactoryException
	 *             An error occur during initialization.
	 */

	protected void initFactory(ServletContext servletContext, String proposedFilename) throws DefinitionsFactoryException, FileNotFoundException {

		// Init list of filenames
		StringTokenizer tokenizer = new StringTokenizer(proposedFilename, ",");
		this.filenames = new ArrayList(tokenizer.countTokens());
		while (tokenizer.hasMoreTokens()) {
			this.filenames.add(tokenizer.nextToken().trim());
		}

		loaded = new HashMap();
		defaultFactory = createDefaultFactory(servletContext);
		// if (log.isDebugEnabled())
		// log.debug("default factory:" + defaultFactory);
	}

	/**
	 * Get default factory.
	 * 
	 * @return Default factory
	 */
	protected DefinitionsFactory getDefaultFactory() {
		return defaultFactory;
	}

	/**
	 * Create default factory .
	 * 
	 * @param servletContext
	 *            Current servlet context. Used to open file.
	 * @return Created default definition factory.
	 * @throws DefinitionsFactoryException
	 *             If an error occur while creating factory.
	 * @throws FileNotFoundException
	 *             if factory can't be loaded from filenames.
	 */
	protected DefinitionsFactory createDefaultFactory(ServletContext servletContext) throws DefinitionsFactoryException, FileNotFoundException {

		XmlDefinitionsSet rootXmlConfig = parseXmlFiles(servletContext, "", null);

		if (rootXmlConfig == null) {
			throw new FileNotFoundException();
		}

		rootXmlConfig.resolveInheritances();
		return new DefinitionsFactory(rootXmlConfig);
	}

	/**
	 * Extract key that will be used to get the sub factory.
	 * 
	 * @param name
	 *            Name of requested definition
	 * @param request
	 *            Current servlet request.
	 * @param servletContext
	 *            Current servlet context
	 * @return the key or null if not found.
	 * @roseuid 3AF6F887018A
	 */
	protected Object getDefinitionsFactoryKey(String name, ServletRequest request, ServletContext servletContext) {
		Object key = null;

		HttpSession session = ((HttpServletRequest) request).getSession(false);
		if (session != null) {
			key = session.getAttribute(FACTORY_SELECTOR_KEY);
		}

		return key;
	}

	/**
	 * Create a factory for specified key. If creation failed, return default factory, and output an error message in console.
	 * 
	 * @param key
	 * @return Definition factory for specified key.
	 * @throws DefinitionsFactoryException
	 *             If an error occur while creating factory.
	 */

	protected DefinitionsFactory createFactory(Object key, ServletRequest request, ServletContext servletContext) throws DefinitionsFactoryException {

		if (key == null) {
			return getDefaultFactory();
		}

		// Already loaded ?
		DefinitionsFactory factory = (DefinitionsFactory) loaded.get(key);
		if (factory != null) { // yes, stop loading
			return factory;
		}

		// Try to load file associated to key. If fail, stop and return default
		// factory.
		XmlDefinitionsSet lastXmlFile = parseXmlFiles(servletContext, "_" + (String) key, null);

		if (lastXmlFile == null) {
			log.warn("No definition factory associated to key '" + key + "'. Use default factory instead.");

			factory = getDefaultFactory();
			loaded.put(key, factory);
			return factory;
		}

		// Parse default file, and add key file.
		XmlDefinitionsSet rootXmlConfig = parseXmlFiles(servletContext, "", null);

		rootXmlConfig.extend(lastXmlFile);
		rootXmlConfig.resolveInheritances();

		factory = new DefinitionsFactory(rootXmlConfig);
		loaded.put(key, factory);

		log.info(factory);

		// return last available found !
		return factory;
	}

	protected XmlDefinitionsSet parseXmlFiles(ServletContext servletContext, String postfix, XmlDefinitionsSet xmlDefinitions) throws DefinitionsFactoryException {

		if (postfix != null && postfix.length() == 0) {
			postfix = null;
		}

		// Iterate throw each file name in list
		Iterator i = filenames.iterator();
		while (i.hasNext()) {
			String filename = concatPostfix((String) i.next(), postfix);
			xmlDefinitions = parseXmlFile(servletContext, filename, xmlDefinitions);
		}

		return xmlDefinitions;
	}

	protected XmlDefinitionsSet parseXmlFile(ServletContext servletContext, String filename, XmlDefinitionsSet xmlDefinitions) throws DefinitionsFactoryException {

		try {
			InputStream input = servletContext.getResourceAsStream(filename);
			// Try to load using real path.
			// This allow to load config file under websphere 3.5.x
			// Patch proposed Houston, Stephen (LIT) on 5 Apr 2002
			if (null == input) {
				try {
					input = new java.io.FileInputStream(servletContext.getRealPath(filename));
				} catch (Exception e) {
				}
			}

			// If the config isn't in the servlet context, try the class loader
			// which allows the config files to be stored in a jar
			if (input == null) {
				input = getClass().getResourceAsStream(filename);
			}

			// If still nothing found, this mean no config file is associated
			if (input == null) {
				if (log.isDebugEnabled()) {
					log.debug("Can't open file '" + filename + "'");
				}
				return xmlDefinitions;
			}

			// Check if parser already exist.
			// Doesn't seem to work yet.
			// if( xmlParser == null )
			if (true) {
				xmlParser = new XmlParser();
				xmlParser.setValidating(isValidatingParser);
			}

			// Check if definition set already exist.
			if (xmlDefinitions == null) {
				xmlDefinitions = new XmlDefinitionsSet();
			}

			xmlParser.parse(input, xmlDefinitions);

		} catch (SAXException ex) {
			if (log.isDebugEnabled()) {
				log.debug("Error while parsing file '" + filename + "'.");
				ex.printStackTrace();
			}
			throw new DefinitionsFactoryException("Error while parsing file '" + filename + "'. " + ex.getMessage(), ex);

		} catch (IOException ex) {
			throw new DefinitionsFactoryException("IO Error while parsing file '" + filename + "'. " + ex.getMessage(), ex);
		}

		return xmlDefinitions;
	}

	/**
	 * Concat postfix to the name. Take care of existing filename extension. Transform the given name "name.ext" to have "name" + "postfix" + "ext".
	 * If there is no ext, return "name" + "postfix".
	 */
	private String concatPostfix(String name, String postfix) {
		if (postfix == null) {
			return name;
		}

		// postfix = "_" + postfix;
		// Search file name extension.
		// take care of Unix files starting with .
		int dotIndex = name.lastIndexOf(".");
		int lastNameStart = name.lastIndexOf(java.io.File.pathSeparator);
		if (dotIndex < 1 || dotIndex < lastNameStart) {
			return name + postfix;
		}

		String ext = name.substring(dotIndex);
		name = name.substring(0, dotIndex);
		return name + postfix + ext;
	}

}
