/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.platform.web.validator;

import java.util.Date;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.validator.GenericTypeValidator;
import org.apache.commons.validator.GenericValidator;

import org.jdesigner.platform.web.ActionInput;
import org.jdesigner.platform.web.ByteGenericValidator;

/**
 * ValidatorSupport
 * 
 * @author leesangboo
 * 
 */
public class ValidatorSupport {

	public static final String FIELD_TEST_NULL = "NULL";

	public static final String FIELD_TEST_NOTNULL = "NOTNULL";

	public static final String FIELD_TEST_EQUAL = "EQUAL";

	/**
	 * 
	 * @param input
	 * @param key
	 * @return
	 */
	public static boolean validateRequired(ActionInput input, String key) {
		String value = input.getString(key);
		return !GenericValidator.isBlankOrNull(value);
	}

	/**
	 * 
	 * @param input
	 * @param key
	 * @return
	 */
	public static boolean validateMask(ActionInput input, String key, String mask) {

		String value = input.getString(key);
		if (StringUtils.isEmpty(value)) {
			return true;
		}

		String regex = mask;
		if (regex != null) {
			return value.matches(regex);
		}

		return true;
	}

	/**
	 * 
	 * @param input
	 * @param key
	 * @return
	 */
	public static boolean validateLength(ActionInput input, String key, int minlength, int maxlength) {

		String value = input.getString(key);
		if (StringUtils.isEmpty(value)) {
			return true;
		}

		String max = maxlength + "";
		String min = minlength + "";

		if (max != null) {
			if (value.length() > Integer.parseInt(max)) {
				return false;
			}
		}

		if (min != null) {
			if (value.length() < Integer.parseInt(min)) {
				return false;
			}
		}

		return true;
	}

	/**
	 * 
	 * @param input
	 * @param key
	 * @param minlength
	 * @return
	 */
	public static boolean validateMinLength(ActionInput input, String key, int minlength) {

		String value = input.getString(key);

		if (!GenericValidator.isBlankOrNull(value)) {
			try {
				int min = minlength;

				if (!GenericValidator.minLength(value, min)) {
					return false;
				}
			} catch (Exception e) {
				return false;
			}
		}

		return true;
	}

	/**
	 * 
	 * @param input
	 * @param key
	 * @param maxlength
	 * @return
	 */
	public static boolean validateMaxLength(ActionInput input, String key, int maxlength) {

		String value = input.getString(key);
		if (!GenericValidator.isBlankOrNull(value)) {
			try {
				int max = maxlength;

				if (!GenericValidator.maxLength(value, max)) {
					return false;
				}
			} catch (Exception e) {
				return false;
			}
		}

		return true;
	}

	/**
	 * 
	 * @param input
	 * @param key
	 * @return
	 */
	public static boolean validateByte(ActionInput input, String key) {

		String value = input.getString(key);
		if (!GenericValidator.isBlankOrNull(value)) {
			if (GenericTypeValidator.formatByte(value) == null) {
				return false;
			}
		}
		return true;
	}

	/**
	 * 
	 * @param input
	 * @param key
	 * @return
	 */
	public static boolean validateShort(ActionInput input, String key) {

		String value = input.getString(key);
		if (!GenericValidator.isBlankOrNull(value)) {
			if (GenericTypeValidator.formatShort(value) == null) {
				return false;
			}
		}
		return true;
	}

	/**
	 * 
	 * @param input
	 * @param key
	 * @return
	 */
	public static boolean validateInteger(ActionInput input, String key) {

		String value = input.getString(key);
		if (!GenericValidator.isBlankOrNull(value)) {
			if (GenericTypeValidator.formatInt(value) == null) {
				return false;
			}
		}
		return true;
	}

	/**
	 * 
	 * @param input
	 * @param key
	 * @return
	 */
	public static boolean validateLong(ActionInput input, String key) {
		String value = input.getString(key);
		if (!GenericValidator.isBlankOrNull(value)) {
			if (GenericTypeValidator.formatLong(value) == null) {
				return false;
			}
		}
		return true;
	}

	/**
	 * 
	 * @param input
	 * @param key
	 * @return
	 */
	public static boolean validateFloat(ActionInput input, String key) {
		String value = input.getString(key);
		if (!GenericValidator.isBlankOrNull(value)) {
			if (GenericTypeValidator.formatFloat(value) == null) {
				return false;
			}
		}
		return true;
	}

	/**
	 * 
	 * @param input
	 * @param key
	 * @return
	 */
	public static boolean validateDouble(ActionInput input, String key) {

		String value = input.getString(key);
		if (!GenericValidator.isBlankOrNull(value)) {
			if (GenericTypeValidator.formatDouble(value) == null) {
				return false;
			}
		}
		return true;
	}

	/**
	 * 
	 * @param input
	 * @param key
	 * @param locale
	 * @return
	 */
	public static boolean validateDate(ActionInput input, String key, Locale locale, String datePattern) {

		String value = input.getString(key);
		if (GenericValidator.isBlankOrNull(value)) {
			return true;
		}

		Date result = null;

		try {
			if (datePattern != null && datePattern.length() > 0) {
				result = GenericTypeValidator.formatDate(value, datePattern, false);
			} else {
				result = GenericTypeValidator.formatDate(value, locale);
			}

			return result != null ? true : false;

		} catch (Exception e) {
		}
		return false;
	}

	/**
	 * 
	 * @param input
	 * @param key
	 * @param minrange
	 * @param maxrange
	 * @return
	 */
	public static boolean validateIntRange(ActionInput input, String key, int minrange, int maxrange) {

		String value = input.getString(key);
		if (GenericValidator.isBlankOrNull(value)) {
			return true;
		}

		try {
			int intValue = Integer.parseInt(value);
			int min = minrange;
			int max = maxrange;

			if (!GenericValidator.isInRange(intValue, min, max)) {
				return false;
			}
		} catch (Exception e) {
			return false;
		}

		return true;
	}

	/**
	 * 
	 * @param bean
	 * @param field
	 * @return
	 */
	public static boolean validateFloatRange(ActionInput input, String key, float minrange, float maxrange) {

		String value = input.getString(key);
		if (GenericValidator.isBlankOrNull(value)) {
			return true;
		}

		try {
			float floatValue = Float.parseFloat(value);
			float min = minrange;
			float max = maxrange;

			if (!GenericValidator.isInRange(floatValue, min, max)) {
				return false;
			}
		} catch (Exception e) {
			return false;
		}

		return true;
	}

	/**
	 * 
	 * @param input
	 * @param key
	 * @param minrange
	 * @param maxrange
	 * @return
	 */
	public static boolean validateDoubleRange(ActionInput input, String key, double minrange, double maxrange) {

		String value = input.getString(key);
		if (GenericValidator.isBlankOrNull(value)) {
			return true;
		}

		try {
			double doubleValue = Double.parseDouble(value);
			double min = minrange;
			double max = maxrange;

			if (!GenericValidator.isInRange(doubleValue, min, max)) {
				return false;
			}
		} catch (Exception e) {
			return false;
		}

		return true;
	}

	/**
	 * 
	 * @param input
	 * @param key
	 * @param minrange
	 * @param maxrange
	 * @return
	 */
	public static boolean validateLongRange(ActionInput input, String key, long minrange, long maxrange) {

		String value = input.getString(key);
		if (GenericValidator.isBlankOrNull(value)) {
			return true;
		}

		try {
			long longValue = Long.parseLong(value);
			long min = minrange;
			long max = maxrange;
			if (!GenericValidator.isInRange(longValue, min, max)) {
				return false;
			}
		} catch (Exception e) {
			return false;
		}
		return true;
	}

	/**
	 * 
	 * @param input
	 * @param key
	 * @return
	 */
	public static boolean validateEmail(ActionInput input, String key) {

		String value = input.getString(key);
		if (GenericValidator.isBlankOrNull(value)) {
			return true;
		}

		return GenericValidator.isEmail(value) ? true : false;
	}

	/**
	 * 
	 * @param input
	 * @param key
	 * @param minbytelength
	 * @param charsetName
	 * @return
	 */
	public static boolean validateMinByteLength(ActionInput input, String key, int minbytelength, String charsetName) {

		String value = input.getString(key);
		if (GenericValidator.isBlankOrNull(value)) {
			return true;
		}

		try {
			int min = minbytelength;
			String charset = charsetName;
			if (!ByteGenericValidator.minByteLength(value, min, charset)) {
				return false;
			}
		} catch (Exception e) {
			return false;
		}
		return true;
	}

	/**
	 * 
	 * @param input
	 * @param key
	 * @param maxbytelength
	 * @param charsetName
	 * @return
	 */
	public static boolean validateMaxByteLength(ActionInput input, String key, int maxbytelength, String charsetName) {

		String value = input.getString(key);
		if (GenericValidator.isBlankOrNull(value)) {
			return true;
		}

		try {
			int max = maxbytelength;
			String charset = charsetName;
			if (!ByteGenericValidator.maxByteLength(value, max, charset)) {
				return false;
			}
		} catch (Exception e) {
			return false;
		}
		return true;
	}

}
